import React from 'react';
import { ReactCodeMirrorProps, ReactCodeMirrorRef } from '@uiw/react-codemirror';
import { CodeSandboxProps } from '@uiw/react-codesandbox';
import { CodepenProps } from '@uiw/react-codepen';
import { SplitProps } from '@uiw/react-split';
import './index.less';
export interface CodePreviewProps extends SplitProps {
    prefixCls?: string;
    style?: React.CSSProperties;
    /**
     * To specify a CSS class, use the className attribute.
     */
    className?: string;
    /**
     * Whether to display the border.
     */
    bordered?: boolean;
    /**
     * `JSX` source code
     */
    code?: string;
    /**
     * Whether to display the code interface.
     */
    noCode?: boolean;
    /**
     * Is the background white or plaid?
     */
    bgWhite?: boolean;
    /**
     * Only show Edit
     */
    onlyEdit?: boolean;
    /**
     * Whether to display the preview interface.
     */
    noPreview?: boolean;
    /**
     * Preview area does not display scroll bars
     */
    noScroll?: boolean;
    /**
     * Modify ReactCodemirror props.
     */
    editProps?: ReactCodeMirrorProps;
    /**
     * Dependent component
     */
    dependencies?: Record<string, any>;
    codePenOption?: CodepenProps & {
        /**
         * Packages that do not require comments.
         * @example ['uiw']
         */
        includeModule?: string[];
    };
    codeSandboxOption?: CodeSandboxProps;
    /** @default 'Code' */
    btnText?: string;
    /** @default 'Hide Editor' */
    btnHideText?: string;
    /**
     * `light` / `dark` / `Extension` Defaults to `light`.
     * @default light
     */
    theme?: ReactCodeMirrorProps['theme'];
}
export interface CodePreviewState {
    errorMessage: string;
    fullScreen: boolean;
    width: number | string;
    copied: boolean;
    showEdit: boolean;
}
export interface CodePreviewRef {
    editor?: React.RefObject<ReactCodeMirrorRef>;
    demo?: HTMLDivElement | null;
}
declare const CodePreview: React.ForwardRefExoticComponent<CodePreviewProps & React.RefAttributes<CodePreviewRef>>;
export default CodePreview;
