"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getReactDOMClient = void 0;
exports.useCodePreview = useCodePreview;
var _react = _interopRequireWildcard(require("react"));
var _transform = require("./transform");
var _reactDom = _interopRequireDefault(require("react-dom"));
const getReactDOMClient = () => {
  let _ReactDOM;
  try {
    // 使用 require 解决 react v17 ts 报错问题
    _ReactDOM = require('react-dom/client');
  } catch (err) {
    // console.warn(`如果使用的是react-dom小于v18的版本，可以忽略此警告：${err}`)
  }
  return _ReactDOM;
};
exports.getReactDOMClient = getReactDOMClient;
function useCodePreview(props) {
  const [demoDom, setDemoDom] = (0, _react.useState)();
  const playerId = (0, _react.useRef)("".concat(parseInt(String(Math.random() * 1e9), 10).toString(36)));
  const [fullScreen, setFullScreen] = (0, _react.useState)(false);
  const [errorMessage, setErrorMessage] = (0, _react.useState)('');
  const [showEdit, setShowEdit] = (0, _react.useState)(false);
  const [width, setWidth] = (0, _react.useState)(1);
  const [copied, setCopied] = (0, _react.useState)(false);
  const [code, setCode] = (0, _react.useState)(props.code || '');
  const ReactDOMClient = _react.default.useMemo(() => {
    return window.ReactDOM ? window.ReactDOM : getReactDOMClient();
  }, []);

  /** 通过缓存的方式 解决 react v18 中 的报错   ***/
  // @ts-ignore
  const cachesRef = _react.default.useRef(new Map([]));
  const ReactDOMRender = _ReactDOM => {
    return {
      createRoot: id => {
        return {
          render: render => {
            const caches = cachesRef.current;
            let root = caches.get(id);
            // 存在则不需要重新创建直接进行render操作
            if (root) {
              root.render(render);
            } else {
              // @ts-ignore
              root = _ReactDOM.createRoot(document.getElementById(id));
              root.render(render);
              // 缓存，解决控制台报  ReactDOMClient.createRoot 问题
              caches.set(id, root);
            }
            cachesRef.current = caches;
          }
        };
      }
    };
  };
  /**  ------------------------   ***/

  const executeCode = str => {
    const {
      React: _React,
      ReactDOM: _ReactDOM,
      ReactDOMClient: _ReactDOMClient,
      ...otherDeps
    } = props.dependencies || {};
    let V18ReactDOM = _ReactDOMClient || ReactDOMClient || _ReactDOM || _reactDom.default;
    // 判断是否是 react v18版本
    const isV18 = Reflect.has(V18ReactDOM, 'createRoot');
    const NewReactDOM = isV18 ? ReactDOMRender(V18ReactDOM) : V18ReactDOM;
    try {
      const deps = {
        React: _React || _react.default,
        ...otherDeps,
        ReactDOM: NewReactDOM
      };
      // const args = ['context', 'React', 'ReactDOM', 'Component'];
      const args = [];
      // const argv = [this, React, ReactDOM, Component];
      const argv = [];
      for (const key in deps) {
        args.push(key);
        argv.push(deps[key]);
      }
      if (isV18) {
        // react < v18 中写法替换
        str = str.replace('ReactDOM.render', "ReactDOM.createRoot(\"".concat(playerId.current, "\").render"));
        // react v18 中写法替换
        str = str.replace("ReactDOM.createRoot(_mount_)", "ReactDOM.createRoot(\"".concat(playerId.current, "\")"));
        str = str.replace('_mount_', "");
      } else {
        str = str.replace('_mount_', "document.getElementById('".concat(playerId.current, "')"));
      }
      const input = "".concat(str);
      const {
        code
      } = (0, _transform.babelTransform)(input);
      args.push(code || '');
      // console.log('code:', argv)
      // eslint-disable-next-line no-new-func
      new Function(...args).apply(null, argv);
      setErrorMessage('');
    } catch (err) {
      let message = '';
      if (err && err.message) {
        message = err.message;
      } else {
        message = JSON.stringify(err);
      }
      setErrorMessage(message);
    }
  };
  // eslint-disable-next-line react-hooks/exhaustive-deps
  (0, _react.useEffect)(() => executeCode(code), [code, props.noPreview, demoDom]);
  return {
    playerId,
    demoDom,
    setDemoDom,
    fullScreen,
    setFullScreen,
    errorMessage,
    setErrorMessage,
    width,
    setWidth,
    showEdit,
    setShowEdit,
    copied,
    setCopied,
    code,
    setCode,
    executeCode
  };
}